@@ -48,6 +48,8 @@
 #include "defs.h"
 #include "lib.h"
 #include "driver.h"
+#include <linux/proc_fs.h>
+#include <linux/seq_file.h>
 
 #ifndef HZ
 # error HZ is not defined...
@@ -85,11 +87,7 @@ static volatile unsigned long	rx_flag			= 0;
 static volatile unsigned long	resetting_ctrl		= 0;
 int				card_config;
 static struct work_struct	closing_work;
-#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,20)
 static void			closing_worker (struct work_struct *);
-#else
-static void			closing_worker (void *);
-#endif
 static volatile unsigned long	closing_worker_running	= 0;
 static void		     (* close_func) (void *)	= NULL;
 static void *			close_data		= NULL;
@@ -98,13 +96,8 @@ static DECLARE_WAIT_QUEUE_HEAD(wait);
 static DECLARE_WAIT_QUEUE_HEAD(capi_wait);
 static DECLARE_WAIT_QUEUE_HEAD(tx_wait);
 static DECLARE_WAIT_QUEUE_HEAD(rx_wait);
-#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,23)
 static DECLARE_COMPLETION(hotplug);
 static DECLARE_COMPLETION(notify);
-#else
-static DECLARE_MUTEX_LOCKED(hotplug);
-static DECLARE_MUTEX_LOCKED(notify);
-#endif
 
 #define	SCHED_WAKEUP_CAPI	{ atomic_set (&thread_capi_flag, 1); wake_up_interruptible (&capi_wait); }
 #define	SCHED_WAKEUP		{ atomic_set (&got_kicked, 1); wake_up_interruptible (&wait); }
@@ -200,14 +193,8 @@ static inline const char * name_of (int code) {
 
 /*---------------------------------------------------------------------------*\
 \*---------------------------------------------------------------------------*/
-#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,22)
 static void select_completion (struct urb * purb) {
 	struct usb_ctrlrequest	* req;
-#else
-static void select_completion (struct urb * purb, struct pt_regs * ptr) {
-	struct usb_ctrlrequest	* req;
-	UNUSED_ARG (ptr);
-#endif
 	assert (NULL != purb);
 	req = (struct usb_ctrlrequest *) purb->context;
 	LOG(
@@ -273,14 +260,8 @@ int select_config (
 
 /*---------------------------------------------------------------------------*\
 \*---------------------------------------------------------------------------*/
-#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,22)
 static void reset_completion (struct urb * purb) {
 	struct usb_ctrlrequest *	req;
-#else
-static void reset_completion (struct urb * purb, struct pt_regs * ptr) {
-	struct usb_ctrlrequest *	req;
-	UNUSED_ARG (ptr);
-#endif
 	assert (NULL != purb);
 	info (purb->status == 0);
 
@@ -386,6 +367,7 @@ static void kill_version (card_p card) {
 	nvers = 0;
 } /* kill_version */
 
+#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,34)
 /*---------------------------------------------------------------------------*\
 \*---------------------------------------------------------------------------*/
 static void pprintf (char * page, int * len, const char * fmt, ...) {
@@ -395,6 +377,7 @@ static void pprintf (char * page, int * len, const char * fmt, ...) {
 	*len += vsnprintf (page + *len, PPRINTF_LEN, fmt, args);
 	va_end (args);
 } /* pprintf */
+#endif
 
 /*---------------------------------------------------------------------------*\
 \*---------------------------------------------------------------------------*/
@@ -586,6 +569,7 @@ static char * __kcapi proc_info (struct capi_ctr * ctrl) {
 	return text;
 } /* proc_info */
 
+#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,34)
 /*---------------------------------------------------------------------------*\
 \*---------------------------------------------------------------------------*/
 static int __kcapi ctr_info (
@@ -644,6 +628,76 @@ static int __kcapi ctr_info (
 	*start = page - ofs;
 	return ((count < len - ofs) ? count : len - ofs);
 } /* ctr_info */
+#endif
+
+#if LINUX_VERSION_CODE > KERNEL_VERSION(2,6,33)
+/*---------------------------------------------------------------------------*\
+\*---------------------------------------------------------------------------*/
+static int __kcapi ctr_info (
+	struct seq_file  *mp,
+	void    *vp
+) {
+	struct capi_ctr  *ctrl = mp->private;
+	card_p			card;
+	char *			temp;
+	unsigned char   flag;
+
+	assert (ctrl);
+	card = (card_p) ctrl->driverdata;
+#if defined (__fxusb__)
+	assert (card->dc <= MAX_DEVICE_CODE);
+	seq_printf(mp, "%-16s %s (%s)\n", "name", SHORT_LOGO, name_of (card->dc));
+#else
+	seq_printf(mp, "%-16s %s\n", "name", SHORT_LOGO);
+#endif
+	seq_printf(mp, "%-16s %d\n", "dev", card->dev->devnum);
+	temp = card->version ? card->string[1] : "A1";
+	seq_printf(mp, "%-16s %s\n", "type", temp);
+	temp = card->version ? card->string[0] : "-";
+	seq_printf(mp, "%-16s %s\n", "ver_driver", temp);
+	seq_printf(mp, "%-16s %s\n", "ver_cardtype", SHORT_LOGO);
+
+	flag = ((unsigned char *) (ctrl->profile.manu))[3];
+	if (flag) {
+		seq_printf(mp, "%-16s%s%s%s%s%s%s%s\n", "protocol",
+			(flag & 0x01) ? " DSS1" : "",
+			(flag & 0x02) ? " CT1" : "",
+			(flag & 0x04) ? " VN3" : "",
+			(flag & 0x08) ? " NI1" : "",
+			(flag & 0x10) ? " AUSTEL" : "",
+			(flag & 0x20) ? " ESS" : "",
+			(flag & 0x40) ? " 1TR6" : ""
+		);
+	}
+	flag = ((unsigned char *) (ctrl->profile.manu))[5];
+	if (flag) {
+		seq_printf(mp, "%-16s%s%s%s%s\n", "linetype",
+			(flag & 0x01) ? " point to point" : "",
+			(flag & 0x02) ? " point to multipoint" : "",
+			(flag & 0x08) ? " leased line without D-channel" : "",
+			(flag & 0x04) ? " leased line with D-channel" : ""
+		);
+	}
+	return 0;
+} /* ctr_info */
+#endif
+
+#if LINUX_VERSION_CODE > KERNEL_VERSION(2,6,33)
+/*---------------------------------------------------------------------------*\
+\*---------------------------------------------------------------------------*/
+static int ctr_proc_open(struct inode *inode, struct file *file)
+{
+       return single_open(file, ctr_info, PDE(inode)->data);
+}
+
+const struct file_operations ctr_proc_fops = {
+       .owner          = THIS_MODULE,
+       .open           = ctr_proc_open,
+       .read           = seq_read,
+       .llseek         = seq_lseek,
+       .release        = single_release,
+};
+#endif
 
 /*---------------------------------------------------------------------------*\
 \*---------------------------------------------------------------------------*/
@@ -667,7 +721,11 @@ static int kcapi_init (struct usb_device * dev) {
 	ctrl->release_appl =	release_appl;
 	ctrl->send_message =	send_msg;
 	ctrl->procinfo =	proc_info;
+#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,34)
 	ctrl->ctr_read_proc =	ctr_info;
+#else
+	ctrl->proc_fops =	&ctr_proc_fops;
+#endif
 	if (0 != attach_capi_ctr (ctrl)) {
 		ERROR("Could not attach the controller.\n");
 		return -EBUSY;
@@ -718,13 +776,8 @@ int __kcapi add_card (struct usb_device * dev) {
 
 /*---------------------------------------------------------------------------*\
 \*---------------------------------------------------------------------------*/
-#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,20)
 static void closing_worker (struct work_struct *work) {
 	card_p	card = capi_card;
-#else
-static void closing_worker (void * ctx) {
-	card_p	card = (card_p) ctx;
-#endif
 
 	DECLARE_WAIT_QUEUE_HEAD(close_wait);
 
@@ -820,11 +873,7 @@ void start_closing_worker (void (* func) (void *), void * data) {
 	close_func = func;
 	close_data = data;
 
-#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,20)
 	INIT_WORK (&closing_work, closing_worker);
-#else
-	INIT_WORK (&closing_work, closing_worker, capi_card);
-#endif
 	schedule_work (&closing_work);
 	LOG("Worker scheduled.\n");
 } /* start_closing_worker */
@@ -1020,24 +1069,14 @@ static void issue_hard_error (card_p pdc, const char * msg, int arg) {
 
 /*---------------------------------------------------------------------------*\
 \*---------------------------------------------------------------------------*/ 
-#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,22)
 static void usb_write_completion (struct urb * purb) {
-#else
-static void usb_write_completion (struct urb * purb, struct pt_regs * ptr) {
-	UNUSED_ARG (ptr);
-#endif
 	UNUSED_ARG (purb);
 	tasklet_schedule (&tx_tasklet);
 } /* usb_write_completion */
 
 /*---------------------------------------------------------------------------*\
 \*---------------------------------------------------------------------------*/ 
-#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,22)
 static void usb_read_completion (struct urb * purb) {
-#else
-static void usb_read_completion (struct urb * purb, struct pt_regs * ptr) {
-	UNUSED_ARG (ptr);
-#endif
 	UNUSED_ARG (purb);
 	tasklet_schedule (&rx_tasklet);
 } /* usb_read_completion */
@@ -1256,11 +1295,7 @@ static int scheduler (void * arg) {
 		}
 	}
 	LOG("Scheduler thread stopped.\n");
-#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,23)
 	complete (&hotplug);
-#else
-	up (&hotplug);
-#endif
 	return 0;
 } /* scheduler */
 
@@ -1289,11 +1324,7 @@ void kill_thread (void) {
 #if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,30)
 		if (NULL == pid_task ( find_vpid(thread_pid), PIDTYPE_PID )) {
 #else
-#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,25)
 		if (NULL == find_task_by_vpid (thread_pid)) {
-#else
-		if (NULL == find_task_by_pid (thread_pid)) {
-#endif
 #endif
 			LOG("Thread[%d] has died before!\n", thread_pid);
 		} else {
@@ -1303,11 +1334,7 @@ void kill_thread (void) {
 				SCHED_WAKEUP;
 			}
 			LOG("Scheduler thread signalled, waiting...\n");
-#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,23)
 			wait_for_completion (&hotplug);
-#else
-			down (&hotplug);
-#endif
 			LOG("Scheduler thread[%d] terminated.\n", thread_pid);
 		}
 		thread_pid = -1;
